@@ -53,11 +53,11 @@
 #'    all slope coefficients are jointly zero, all times dummies and slope
 #'    coefficients are jointly zero.
 #'
+#' @param object An object of class `pdynmc`.
 #' @param param A character string that denotes the null hypothesis. Choices are
 #'    time.dum (i.e., all time dummies are jointly zero), slope (i.e., all slope
 #'    coefficients are jointly zero), and all (i.e., all dummies and slope
 #'    coefficients are jointly zero).
-#' @param object An object of class `pdynmc`.
 #' @return An object of class `htest` which contains the F test statistic and
 #'    corresponding p-value for the tested null hypothesis.
 #'
@@ -115,20 +115,32 @@
 #'     include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, varname.dum = "year",
 #'     w.mat = "iid.err", std.err = "corrected", estimation = "onestep",
 #'     opt.meth = "none")
-#'  wald.fct(param = "all", m1)
+#'  wald.fct(m1, param = "all")
 #' }
 #' }
 #'
 #'
 wald.fct 		<- function(
- param
- ,object
+ object
+ ,param
 ){
 
   if(!inherits(object, what = "pdynmc")){
     stop("Use only with \"pdynmc\" objects.")
   }
 
+  if(param == "all"){
+    params <- "time dummy and/or slope coefficient"
+  } else{
+    if(param == "time.dum"){
+      params <- "time dummy"
+    } else{
+      if(param == "slope"){
+        params <- "slope coefficient"
+      }
+    }
+  }
+
   coef.est				<- ifelse((sapply(get(paste("step", object$iter, sep = ""), object$par.optim), FUN = is.na)),
 						yes = get(paste("step", object$iter, sep = ""), object$par.clForm),
 						no = get(paste("step", object$iter, sep = ""), object$par.optim) )
@@ -186,7 +198,8 @@ wald.fct 		<- function(
   names(dof)	<- "df"
   pval		<- stats::pchisq(w.stat, df = dof, lower.tail = FALSE)
   wald		<- list(statistic = w.stat, p.value = pval, parameter = dof, method = "Wald test"
-				,data.name = paste(object$iter, "step GMM Estimation; H0: ", param, " parameters are jointly zero", sep = "")
+				,data.name = paste(object$iter, "step GMM Estimation", sep = "")
+				,alternative = paste("at least one ",  params," is not equal to zero", sep = "")
 				)
   class(wald) <- "htest"
   return(wald)
@@ -329,7 +342,7 @@ jtest.fct		<- function(
 
   coef.est		<- ifelse((sapply(get(paste("step", object$iter, sep = ""), object$par.optim), FUN = is.na)), yes = get(paste("step", object$iter, sep = ""), object$par.clForm), no = get(paste("step", object$iter, sep = ""), object$par.optim) )
   Szero.j		<- get(paste("step", object$iter, sep = ""), object$residuals.int)
-  Z.temp		<- object$data$Z
+  Z.temp		<- object$data$Z.temp
   W.j			<- get(paste("step", object$iter, sep = ""), object$w.mat)
   n.inst		<- object$data$n.inst
 
@@ -345,7 +358,8 @@ jtest.fct		<- function(
   method			<- "J-Test of Hansen"
   pval			<- stats::pchisq(stat, df = param, lower.tail = FALSE)
   jtest			<- list(statistic = stat, p.value = pval, parameter = param, method = method
-					,data.name = paste(object$iter, "step GMM Estimation; H0: overidentifying restrictions valid", sep = "")
+					,data.name = paste(object$iter, "step GMM Estimation", sep = "")
+					,alternative = paste("overidentifying restrictions invalid", sep = "")
 					)
   class(jtest)		<- "htest"
   return(jtest)
@@ -381,7 +395,8 @@ jtest.fct		<- function(
 
 #' Arellano and Bond Serial Correlation Test.
 #'
-#' \code{mtest.fct} tests for serial correlation in the error terms.
+#' \code{mtest.pdynmc} Methods to test for serial correlation in the error terms
+#'    for objects of class `pdynmc`.
 #'
 #' The null hypothesis is that there is no serial correlation of a
 #'    particular order. The test statistic is computed as proposed by
@@ -389,7 +404,10 @@ jtest.fct		<- function(
 #'    \insertCite{Are2003;textual}{pdynmc}.
 #'
 #' @param object An object of class `pdynmc`.
-#' @param t.order A number denoting the order of serial correlation to test for.
+#' @param order A number denoting the order of serial correlation to test for
+#'    (defaults to `2`).
+#' @param ... further arguments.
+#'
 #' @return An object of class `htest` which contains the Arellano and Bond m test
 #'    statistic and corresponding p-value for the null hypothesis that there is no
 #'    serial correlation of the given order.
@@ -406,7 +424,6 @@ jtest.fct		<- function(
 #' \code{\link{pdynmc}} for fitting a linear dynamic panel data model.
 #'
 #' @references
-#'
 #' \insertAllCited{}
 #'
 #'
@@ -430,7 +447,7 @@ jtest.fct		<- function(
 #'     include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, varname.dum = "year",
 #'     w.mat = "iid.err", std.err = "corrected", estimation = "onestep",
 #'     opt.meth = "none")
-#'  mtest.fct(m1, t.order = 2)
+#'  mtest.fct(m1, order = 2)
 #' }
 #'
 #' \donttest{
@@ -452,14 +469,15 @@ jtest.fct		<- function(
 #'     include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, varname.dum = "year",
 #'     w.mat = "iid.err", std.err = "corrected", estimation = "onestep",
 #'     opt.meth = "none")
-#'  mtest.fct(m1, t.order = 2)
+#'  mtest.fct(m1, order = 2)
 #' }
 #' }
 #'
 #'
 mtest.fct 		<- function(
  object
- ,t.order
+ ,order = 2
+ ,...
 ){
 
   if(!inherits(object, what = "pdynmc")){
@@ -468,7 +486,7 @@ mtest.fct 		<- function(
 
   estimation	<- object$data$estimation
   Szero.j			<- get(paste("step", object$iter, sep = ""), object$residuals.int)
-  Z.temp			<- object$data$Z
+  Z.temp			<- object$data$Z.temp
   vcov.est		<- get(paste("step", object$iter, sep = ""), object$vcov)
   W.j				  <- get(paste("step", object$iter, sep = ""), object$w.mat)
 
@@ -486,7 +504,7 @@ mtest.fct 		<- function(
 
 
   K.t			<- length(varname.dum) - length(varname.dum[!(varname.dum %in% varname.reg)])
-  u.hat.m_o		<- lapply(Szero.j, function(x) c(rep(0, times = t.order), x[1:(length(x)-t.order)]) )
+  u.hat.m_o		<- lapply(Szero.j, function(x) c(rep(0, times = order), x[1:(length(x)-order)]) )
 
   if(estimation == "onestep" & stderr.type == "unadjusted"){
     #    uHtu			<- lapply(lapply(Szero.j, function(x) crossprod(x,x)), function(x) as.numeric(x) * 0.2* H_i.temp * (1/ (n*T - sum(n.inst)+3) ))
@@ -514,8 +532,9 @@ mtest.fct 		<- function(
   stat		<- frac.num/frac.denom
   names(stat)	<- "normal"
   pval		<- 2*stats::pnorm(abs(stat), lower.tail = FALSE)
-  mtest		<- list(statistic = stat, p.value = pval, method = paste("Arellano and Bond (1991) serial correlation test of degree", t.order)
-                 ,data.name = paste(object$iter, "step GMM Estimation; H0: no serial correlation of order ", t.order, " in the error terms", sep = "")
+  mtest		<- list(statistic = stat, p.value = pval, method = paste("Arellano and Bond (1991) serial correlation test of degree", order)
+                 ,data.name = paste(object$iter, "step GMM Estimation", sep = "")
+                 ,alternative = paste("serial correlation of order ", order, " in the error terms", sep = "")
   )
   class(mtest)	<- "htest"
   return(mtest)
